#!/bin/sh
# Update script

# System handshake magic
UNAME=armv5tejl
CODENAME=trophy5
DATESTAMP=20151211013026

FAMILY_ID="$(cut -f2 /proc/navico_platform/family_id)"
if [ "$FAMILY_ID" = "Hobart" ] || [ "$FAMILY_ID" = "Sylvester" ]; then
  MINBOOTVER=-1
else
   MINBOOTVER=4
fi

# This progress bar delay is just used as an approximate speed for moving the progress bar
# After the bar has passed 75%, it will slow to 1/2 this rate, and 1/4 this rate after 90%
PROGRESSBARDELAY=0.75

# Print out the updater version. This needed by the factory to validate the correct version is being
# programmed when doing automated CPU card programming
# NOTE: makeupd.sh will modify this line
echo UPDATEVERSION=Trophy5-1.0-6.0.11-18.hook.violet_6.0.7-0-gcaabf54

DEBUG=${DEBUG:-false}
if [ -e /dev/shm/debug ]; then
   DEBUG=true
fi
${DEBUG} && set -x

# If anything goes awry, exit early.
set -e

# We expect to be invoked from the /tmp directory.
# We expect to have working /proc and /sys directories, as well as a populated /dev

### This upgrade script expects the following environment variables to be present:
# UPDFILE -- a path to the '.upd' update file.
#    ex:  UPDFILE = /path/to/foo.upd
#    This file path is found during the discovery phase of the initramfs
#    and will be relative to the ramfs root.
if [ -z "${UPDFILE}" ]; then
   echo "Internal error: Bad environment.  UPDFILE was not set."
   exit 1;
fi



### This file also encapsulates SHA1 sums of the componets it expects to operate upon.
# These need to be checked before the script proceeds.
# TODO: Use hardware-accelerated SHA1
# TODO: Test the system sha1sum for tampering.
SHA1CMD=sha1sum
sha1check () {
   if [ -z "${1}" ]; then
      echo "no file to check, aborting"
      return 1;
   fi

   local EXTERNAL=0
   if [ "${1}" = "-external" ]; then
      EXTERNAL=1
      shift 1
   fi

   THISSHA1="$(
      grep -A 500 "^###SHA1MAGIC###$" "${0}" | \
         grep -o "[^ ]\{40\}  ${1}$"
   )"
   if [ "$?" -ne "0" ]; then
      echo "File \"${1}\" not found in internal hash table."
      return 1
   fi
   THISSHA1="$(echo "${THISSHA1}" | cut -f 1 -d ' ' )"
   ${DEBUG} && echo "Stored SHA1 for ${1} is ${THISSHA1}"

   if [ "${EXTERNAL}" = 1 ]; then
      if [ ! -r "$(dirname "${UPDFILE}")/${1}" ]; then
         echo "Couldn't open external file ${1}"
         return 1
      fi
      LOCALSHA1="$("${SHA1CMD}" "$(dirname "${UPDFILE}")/${1}" | cut -f 1 -d ' ')"
   else
      LOCALSHA1="$(tar -xOf ${UPDFILE} ${1} | "${SHA1CMD}" | cut -f 1 -d ' ' )"
   fi

   ${DEBUG} && echo "Calculated SHA1 for ${1} is ${LOCALSHA1}"

   if [ "${THISSHA1}" != "${LOCALSHA1}" ]; then
      echo "ERROR: Update corrupt.  File \"${1}\" failed checksum."
      echo "Please delete this update and replace it with a new copy."
      return 1;
   else
      ${DEBUG} && echo "DEBUG: successful sha1sum check: ${1}"
   fi
   return 0;
}

# Taken from zzz_imx:/usr/libexec/updater/selectupdate
get_last_update()
{
   local mnt=/mnt/home
   mkdir -p $mnt

   if mount -o ro $mnt ; then
      if [ -r $mnt/update-history.txt ]; then
	  # get datestamp:filename of last update, if successful
          # substitute 0 for missing datestamp (when NF==2)
	  awk -F$'\t' 'END {
                           if ($NF==0) {
                               printf "%s:%s\n",  (NF>=3 ? $2 : 0), $1
                           }
                       }' < $mnt/update-history.txt
      fi

      umount $mnt
   fi
   rmdir $mnt || true
}

# Taken from zzz_imx:/usr/libexec/updater/selectupdate: to_runupdate()
log_update_pending()
{
   local updatefile="$1"
   local datestamp="$2"

   local mnt=/mnt/home
   mkdir -p $mnt

   if mount $mnt ; then
      # mark as pending for now
      printf "%s\t%s\tP\n" "$(basename "${updatefile}")" "$datestamp" >> $mnt/update-history.txt
      sync
      umount $mnt
   fi
   rmdir $mnt || true
}

# Taken from zzz_imx:/sbin/update where the return code from runupdate is logged
log_update_result() {
   local result=$1
   local mnt=/mnt/home
   mkdir -p $mnt

   if mount $mnt ; then
      if [ -e $mnt/update-history.txt ]; then
	  sed -i "$ s/^\(.*\)\tP$/\1\t${result}/" $mnt/update-history.txt
	  sync
      fi
      umount $mnt
   else
      echo "Failed to log update result" >&2
   fi
   rmdir $mnt || true
}

case "$CODENAME" in
houdini)
   # if rescue kernel, install anyway (old behaviour)
   if grep -q "rescue" /proc/cmdline; then
      echo "Running latest updater - rescue kernel" >&2
      log_update_pending "$UPDFILE" "$DATESTAMP"
   else
      # datestamp:filename of last update, if successful
      last_update=$(get_last_update)

      # skip candidate unless name or date different from previous
      if [ "$last_update" != "${DATESTAMP}:$(basename "${UPDFILE}")" ]; then
         echo "Running latest updater" >&2
         log_update_pending "$UPDFILE" "$DATESTAMP"
      else
         echo "Skipping update to same version" >&2
         exit 0
      fi
   fi
   ;;
*)
   ;;
esac

sha1check libupdate.sh || exit 1

tar -xf "${UPDFILE}" libupdate.sh

. ./libupdate.sh

start_splash_screen

# Background progress loop.
PROGRESS=0
(  set +x
   while [ ! -f /stop-progress-flag.$$ ] && \
      [ ${PROGRESS} -le 98 ];
   do
      PROGRESS=$((${PROGRESS}+1))
      display_progress ${PROGRESS}
      sleep ${PROGRESSBARDELAY}
      if [ ${PROGRESS} -ge 75 ]; then
         # 2x delay
         sleep ${PROGRESSBARDELAY}
      fi
      if [ ${PROGRESS} -ge 90 ]; then
         # 4x delay
         sleep ${PROGRESSBARDELAY}
         sleep ${PROGRESSBARDELAY}
      fi
   done
) &


###  Determine unit type

# What payloads are we carrying?
# Search our signature table.
COMPONENTLIST="$( \
   grep -A 42 -m 1 '^###SHA1MAGIC###$' "${0}" | \
   sed 's/[^ ]\{40\}  //; /###SHA1MAGIC###/d' | \
   sort | \
   uniq
)"

update_corrupt() {
   echo "ERROR:  This update is damaged.  Halted!"
   display_error_image
   while true; do
      sleep 42d
   done
}

# Check all the components are valid before doing anything
for COMPONENT in $COMPONENTLIST; do
   sha1check $COMPONENT || update_corrupt
done

cat /proc/mtd

# Repair any units that did not get config partition generated correctly in the factory
check_repair_config_parition

# Do the root file-system first, so we get the tools on there for later use
if [ ! -z "$(echo ${COMPONENTLIST} | grep root.tar.xz)" ]; then
   # Delete and re-generate default file-system
   ubiupdatevol -t /dev/$(find_ubi_volume_num_from_name root)
   mount /mnt

   tar -xOf "${UPDFILE}" root.tar.xz | xzcat | tar -x -C /mnt

   umount /mnt
   sync
fi

mount /mnt
mount --bind /dev /mnt/dev
mount --bind /sys /mnt/sys

chroot /mnt sh -l -c "ldconfig"

if [ ! -z "$(echo ${COMPONENTLIST} | grep kernel.bin)" ]; then
   update_mtd_image kernel kernel.bin
fi

if [ ! -z "$(echo ${COMPONENTLIST} | grep boot.bin)" ]; then
   update_boot boot.bin $MINBOOTVER
fi

FBSPLASH_SIZE=$(cut -f 2 /proc/navico_platform/disp_size | cut -c0)

case "$CODENAME" in
sylvester4g)
   FBSPLASH_SIZE="${FBSPLASH_SIZE}g"
   ;;
*)
   ;;
esac

if [ ! -z "$(echo ${COMPONENTLIST} | grep logo${FBSPLASH_SIZE}.bin)" ]; then
   update_mtd_image logo logo${FBSPLASH_SIZE}.bin
fi

umount /mnt/sys
umount /mnt/dev
umount /mnt

check_updated_skel_files

echo "Update successful"

touch /stop-progress-flag.$$
sleep 1
display_progress 100
sleep 1

display_done_image

case "$CODENAME" in
houdini)
   reboot_on_exit
   log_update_result 0
   ;;
*)
   wait_for_media_reboot
   ;;
esac

# Done!
exit 0
exit 0
###SHA1MAGIC###
4006ef11ee461549727fefa1fd3cdfb4d9a2bc86  libupdate.sh
2903c824f4ec07c448852ea8687a402804235d58  boot.bin
81b1b73b1a532f06e327497b44e9093691e6e101  kernel.bin
67191e4033dbabd6a83743982a45d6d2f0e34c0d  root.tar.xz
5ab2303edaf3815daea1d54739ac3a03fef4d44a  fbsplash.tar.xz
