#!/bin/sh

# This script is used to determine if an updater is valid for the system upon
# which it is being run.
#
# COMMAND LINE
#
# One argument, a path to the file that needs testing
#
# RETURN CODE
#
# The return code is 0 if the script is successful, non-0 otherwise.
#
# OUTPUT
#
# The string 'true' is written to stdout, if the updater is valid.
# The string 'false: {reason}' is written to sdtout, if the updater is not valid.
# The string 'error: {reason}' is written to stderr, if the script failed.
#
# NOTE: This code must be compatible with the initrd code that is used to
# validate updaters, see /usr/libexec/verifysystem in initrd-updater

verify() {
  if [ -z "$1" ]; then
    echo "error: missing argument" 1>&2
    return 1
  elif [ ! -r "$1" ]; then
    echo "error: unreadable file: '$1'" 1>&2
    return 1
  else
    local mycodename="$( \
      sed -n 's/.*navico_sku\.codename=\([^ ]*\).*/\1/p' /proc/cmdline | \
      tr '[:upper:]' '[:lower:]' \
    )"

    if [ -z "$mycodename" ] && [ -r /proc/navico_platform/family_id ]; then
      mycodename="$(cut -f 2 /proc/navico_platform/family_id | tr '[:upper:]' '[:lower:]')"
    fi

    if [ "$mycodename" = 'forefraz' ]; then
       if [ "$(cut -f2 /proc/navico_platform/sonar_present)" = "true" ]; then
          mycodename=foreman
       else
          mycodename=frazier
       fi
    fi

    case $(sed -ne 's/^Hardware\s*: //p' /proc/cpuinfo) in
        *" i.MX 6 "*) mycodename="${mycodename}:mx6" ;; # "Freescale i.MX 6 NAVICO Board"
    esac

    if [ -z "$mycodename" ]; then
      echo 'error: missing host codename' 1>&2
      return 1
    fi

    local myuname="$(uname -m)"

    rm -f /tmp/update.sh
    if ! tar -C /tmp -xf "$1" update.sh 2>/dev/null; then
      echo 'error: not an updater' 1>&2
      return 1
    fi

    local updcodename="$( \
      cut -d'#' -f1 /tmp/update.sh | \
      grep -m1 -o "CODENAME=[^ ]*" | \
      cut -d'=' -f2 )"

    if [ "$updcodename" != "$mycodename" ]; then
      local updcodenamelist="$( \
        cut -d'#' -f1 /tmp/update.sh | \
        grep -m1 -o "CODENAMELIST=[^ ]*" | \
        cut -d'=' -f2 )"

      local IFS=,
      for updcodename in $updcodenamelist; do
        if [ "$updcodename" = "$mycodename" ]; then
          break
        fi
      done
      unset IFS
    fi

    local upduname="$( \
      cut -d'#' -f1 /tmp/update.sh | \
      grep -m1 -o "UNAME=[^ ]*" | \
      cut -d'=' -f2 )"

    rm -f /tmp/update.sh

    if [ "$updcodename" != "$mycodename" ] ; then
      echo 'false: codename mismatch'
      return 0
    elif [ "$upduname" != "$myuname" ]; then
      echo 'false: hardware mismatch'
      return 0
    fi
  fi # -r "$1"

  echo 'true'
}

verify "$1"
exit $?
